// This file was autogenerated.

#ifndef CLAZY_ANCHOR_HEADER_H
#define CLAZY_ANCHOR_HEADER_H

extern volatile int ClazyAnchor_ConnectNonSignal;
extern volatile int ClazyAnchor_ConnectNotNormalized;
extern volatile int ClazyAnchor_ContainerAntiPattern;
extern volatile int ClazyAnchor_LambdaInConnect;
extern volatile int ClazyAnchor_LambdaUniqueConnection;
extern volatile int ClazyAnchor_MutableContainerKey;
extern volatile int ClazyAnchor_QColorFromLiteral;
extern volatile int ClazyAnchor_QDateTimeUtc;
extern volatile int ClazyAnchor_Qenums;
extern volatile int ClazyAnchor_QFileInfoExists;
extern volatile int ClazyAnchor_QGetEnv;
extern volatile int ClazyAnchor_QMapWithPointerKey;
extern volatile int ClazyAnchor_StringArg;
extern volatile int ClazyAnchor_QStringInsensitiveAllocation;
extern volatile int ClazyAnchor_StringRefCandidates;
extern volatile int ClazyAnchor_QtMacros;
extern volatile int ClazyAnchor_QVariantTemplateInstantiation;
extern volatile int ClazyAnchor_StrictIterators;
extern volatile int ClazyAnchor_TemporaryIterator;
extern volatile int ClazyAnchor_UnusedNonTrivialVariable;
extern volatile int ClazyAnchor_WritingToTemporary;
extern volatile int ClazyAnchor_WrongQGlobalStatic;
extern volatile int ClazyAnchor_AutoUnexpectedQStringBuilder;
extern volatile int ClazyAnchor_ChildEvent_qobject_cast;
extern volatile int ClazyAnchor_Connect3argLambda;
extern volatile int ClazyAnchor_ConstSignalOrSlot;
extern volatile int ClazyAnchor_DetachingTemporary;
extern volatile int ClazyAnchor_Foreach;
extern volatile int ClazyAnchor_IncorrectEmit;
extern volatile int ClazyAnchor_InefficientQListSoft;
extern volatile int ClazyAnchor_InstallEventFilter;
extern volatile int ClazyAnchor_NonPodStatic;
extern volatile int ClazyAnchor_OverriddenSignal;
extern volatile int ClazyAnchor_PostEvent;
extern volatile int ClazyAnchor_QDeleteAll;
extern volatile int ClazyAnchor_qhash_namespace;
extern volatile int ClazyAnchor_QLatin1StringNonAscii;
extern volatile int ClazyAnchor_QPropertyWithoutNotify;
extern volatile int ClazyAnchor_QStringLeft;
extern volatile int ClazyAnchor_RangeLoop;
extern volatile int ClazyAnchor_ReturningDataFromTemporary;
extern volatile int ClazyAnchor_RuleOfTwoSoft;
extern volatile int ClazyAnchor_VirtualSignal;
extern volatile int ClazyAnchor_BaseClassEvent;
extern volatile int ClazyAnchor_ContainerInsideLoop;
extern volatile int ClazyAnchor_CopyablePolymorphic;
extern volatile int ClazyAnchor_CtorMissingParentArgument;
extern volatile int ClazyAnchor_FunctionArgsByRef;
extern volatile int ClazyAnchor_FunctionArgsByValue;
extern volatile int ClazyAnchor_GlobalConstCharPointer;
extern volatile int ClazyAnchor_ImplicitCasts;
extern volatile int ClazyAnchor_MissingQ_OBJECT;
extern volatile int ClazyAnchor_MissingTypeinfo;
extern volatile int ClazyAnchor_OldStyleConnect;
extern volatile int ClazyAnchor_QStringAllocations;
extern volatile int ClazyAnchor_ReserveCandidates;
extern volatile int ClazyAnchor_ReturningVoidExpression;
extern volatile int ClazyAnchor_RuleOfThree;
extern volatile int ClazyAnchor_VirtualCallsFromCTOR;
extern volatile int ClazyAnchor_AssertWithSideEffects;
extern volatile int ClazyAnchor_UnneededCast;
extern volatile int ClazyAnchor_DetachingMember;
extern volatile int ClazyAnchor_ThreadWithSlots;

int clazy_dummy()
{
    return ClazyAnchor_ConnectNonSignal + ClazyAnchor_ConnectNotNormalized + ClazyAnchor_ContainerAntiPattern + ClazyAnchor_LambdaInConnect
        + ClazyAnchor_LambdaUniqueConnection + ClazyAnchor_MutableContainerKey + ClazyAnchor_QColorFromLiteral + ClazyAnchor_QDateTimeUtc + ClazyAnchor_Qenums
        + ClazyAnchor_QFileInfoExists + ClazyAnchor_QGetEnv + ClazyAnchor_QMapWithPointerKey + ClazyAnchor_StringArg + ClazyAnchor_QStringInsensitiveAllocation
        + ClazyAnchor_StringRefCandidates + ClazyAnchor_QtMacros + ClazyAnchor_QVariantTemplateInstantiation + ClazyAnchor_StrictIterators
        + ClazyAnchor_TemporaryIterator + ClazyAnchor_UnusedNonTrivialVariable + ClazyAnchor_WritingToTemporary + ClazyAnchor_WrongQGlobalStatic
        + ClazyAnchor_AutoUnexpectedQStringBuilder + ClazyAnchor_ChildEvent_qobject_cast + ClazyAnchor_Connect3argLambda + ClazyAnchor_ConstSignalOrSlot
        + ClazyAnchor_DetachingTemporary + ClazyAnchor_Foreach + ClazyAnchor_IncorrectEmit + ClazyAnchor_InefficientQListSoft + ClazyAnchor_InstallEventFilter
        + ClazyAnchor_NonPodStatic + ClazyAnchor_OverriddenSignal + ClazyAnchor_PostEvent + ClazyAnchor_QDeleteAll + ClazyAnchor_qhash_namespace
        + ClazyAnchor_QLatin1StringNonAscii + ClazyAnchor_QPropertyWithoutNotify + ClazyAnchor_QStringLeft + ClazyAnchor_RangeLoop
        + ClazyAnchor_ReturningDataFromTemporary + ClazyAnchor_RuleOfTwoSoft + ClazyAnchor_VirtualSignal + ClazyAnchor_BaseClassEvent
        + ClazyAnchor_ContainerInsideLoop + ClazyAnchor_CopyablePolymorphic + ClazyAnchor_CtorMissingParentArgument + ClazyAnchor_FunctionArgsByRef
        + ClazyAnchor_FunctionArgsByValue + ClazyAnchor_GlobalConstCharPointer + ClazyAnchor_ImplicitCasts + ClazyAnchor_MissingQ_OBJECT
        + ClazyAnchor_MissingTypeinfo + ClazyAnchor_OldStyleConnect + ClazyAnchor_QStringAllocations + ClazyAnchor_ReserveCandidates
        + ClazyAnchor_ReturningVoidExpression + ClazyAnchor_RuleOfThree + ClazyAnchor_VirtualCallsFromCTOR + ClazyAnchor_AssertWithSideEffects
        + ClazyAnchor_UnneededCast + ClazyAnchor_DetachingMember + ClazyAnchor_ThreadWithSlots + 0;
}

#endif
