
#ifndef GL_FRAMEBUFFER_SRB
# define GL_FRAMEBUFFER_SRGB // Warn
#endif

#ifndef GL_FRAMEBUFFER_SRB
# define GL_FRAMEBUFFER_SRB // OK
# define GL_FRAMEBUFFER_SRGB // OK
#endif

#ifndef AA
# define AAA // OK, too small
#endif


#ifndef GL_FRAMEBUFFER_SRGB
# define GL_FRAMEBUFFER_SRGB // OK
#endif
