#include <QtCore/QtCore>
#include <QtCore>
#include <QtGui/QtGui>
#include <QtGui>
#include <QtQml/QtQml>
#include <QtQml>
#include <QtTest/QtTest>
#include <QtTest>
#include <QtNetwork/QtNetwork>
#include <QtNetwork>
#include <QtDBus/QtDBus>
#include <QtDBus>
#include <QtQuick/QtQuick>
#include <QtQuick>
#include <QtQmlModels/QtQmlModels>
#include <QtQmlModels>
#include <QtXml/QtXml>
#include <QtXml>
#include <QtSvg/QtSvg>
#include <QtSvg>
#include <QtConcurrent/QtConcurrent>
#include <QtConcurrent>
#include <QtMultimedia/QtMultimedia>
#include <QtMultimedia>
#include <QtSql/QtSql>
#include <QtSql>
#include <QtPrintSupport/QtPrintSupport>
#include <QtPrintSupport>
#include <QtNetworkAuth/QtNetworkAuth>
#include <QtNetworkAuth>
#include <QtQuickTest>
#include <QtQuickTest/QtQuickTest>




