/*
    SPDX-FileCopyrightText: 2015 Sergio Martins <smartins@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include <filesystem>

int main()
{
    std::filesystem::path path;
    return 0;
}
