#include <QtCore/QAbstractState>
#include <QtCore/QAbstractTransition>
#include <QtCore/QEventTransition>
#include <QtCore/QFinalState>
#include <QtCore/QHistoryState>
#include <QtCore/QSignalTransition>
#include <QtCore/QState>
#include <QtCore/QStateMachine>
#include <QtCore/qabstractstate.h>
#include <QtCore/qabstracttransition.h>
#include <QtCore/qeventtransition.h>
#include <QtCore/qfinalstate.h>
#include <QtCore/qhistorystate.h>
#include <QtCore/qsignaltransition.h>
#include <QtCore/qstate.h>
#include <QtCore/qstatemachine.h>

#include <QtGui/QOpenGLBuffer>
#include <QtGui/QOpenGLDebugLogger>
#include <QtGui/QOpenGLDebugMessage>
#include <QtGui/QOpenGLFramebufferObject>
#include <QtGui/QOpenGLFramebufferObjectFormat>
#include <QtGui/QOpenGLFunctions_1_0>
#include <QtGui/QOpenGLFunctions_1_1>
#include <QtGui/QOpenGLFunctions_1_2>
#include <QtGui/QOpenGLFunctions_1_3>
#include <QtGui/QOpenGLFunctions_1_4>
#include <QtGui/QOpenGLFunctions_1_5>
#include <QtGui/QOpenGLFunctions_2_0>
#include <QtGui/QOpenGLFunctions_2_1>
#include <QtGui/QOpenGLFunctions_3_0>
#include <QtGui/QOpenGLFunctions_3_1>
#include <QtGui/QOpenGLFunctions_3_2_Compatibility>
#include <QtGui/QOpenGLFunctions_3_2_Core>
#include <QtGui/QOpenGLFunctions_3_3_Compatibility>
#include <QtGui/QOpenGLFunctions_3_3_Core>
#include <QtGui/QOpenGLFunctions_4_0_Compatibility>
#include <QtGui/QOpenGLFunctions_4_0_Core>
#include <QtGui/QOpenGLFunctions_4_1_Compatibility>
#include <QtGui/QOpenGLFunctions_4_1_Core>
#include <QtGui/QOpenGLFunctions_4_2_Compatibility>
#include <QtGui/QOpenGLFunctions_4_2_Core>
#include <QtGui/QOpenGLFunctions_4_3_Compatibility>
#include <QtGui/QOpenGLFunctions_4_3_Core>
#include <QtGui/QOpenGLFunctions_4_4_Compatibility>
#include <QtGui/QOpenGLFunctions_4_4_Core>
#include <QtGui/QOpenGLFunctions_4_5_Compatibility>
#include <QtGui/QOpenGLFunctions_4_5_Core>
#include <QtGui/QOpenGLFunctions_ES2>
#include <QtGui/QOpenGLPaintDevice>
#include <QtGui/QOpenGLPixelTransferOptions>
#include <QtGui/QOpenGLShader>
#include <QtGui/QOpenGLShaderProgram>
#include <QtGui/QOpenGLTexture>
#include <QtGui/QOpenGLTextureBlitter>
#include <QtGui/QOpenGLTimeMonitor>
#include <QtGui/QOpenGLTimerQuery>
#include <QtGui/QOpenGLVersionFunctions>
#include <QtGui/QOpenGLVersionProfile>
#include <QtGui/QOpenGLVertexArrayObject>
#include <QtGui/QOpenGLWindow>
#include <QtGui/qopenglbuffer.h>
#include <QtGui/qopengldebug.h>
#include <QtGui/qopenglframebufferobject.h>
#include <QtGui/qopenglfunctions_1_0.h>
#include <QtGui/qopenglfunctions_1_1.h>
#include <QtGui/qopenglfunctions_1_2.h>
#include <QtGui/qopenglfunctions_1_3.h>
#include <QtGui/qopenglfunctions_1_4.h>
#include <QtGui/qopenglfunctions_1_5.h>
#include <QtGui/qopenglfunctions_2_0.h>
#include <QtGui/qopenglfunctions_2_1.h>
#include <QtGui/qopenglfunctions_3_0.h>
#include <QtGui/qopenglfunctions_3_1.h>
#include <QtGui/qopenglfunctions_3_2_compatibility.h>
#include <QtGui/qopenglfunctions_3_2_core.h>
#include <QtGui/qopenglfunctions_3_3_compatibility.h>
#include <QtGui/qopenglfunctions_3_3_core.h>
#include <QtGui/qopenglfunctions_4_0_compatibility.h>
#include <QtGui/qopenglfunctions_4_0_core.h>
#include <QtGui/qopenglfunctions_4_1_compatibility.h>
#include <QtGui/qopenglfunctions_4_1_core.h>
#include <QtGui/qopenglfunctions_4_2_compatibility.h>
#include <QtGui/qopenglfunctions_4_2_core.h>
#include <QtGui/qopenglfunctions_4_3_compatibility.h>
#include <QtGui/qopenglfunctions_4_3_core.h>
#include <QtGui/qopenglfunctions_4_4_compatibility.h>
#include <QtGui/qopenglfunctions_4_4_core.h>
#include <QtGui/qopenglfunctions_4_5_compatibility.h>
#include <QtGui/qopenglfunctions_4_5_core.h>
#include <QtGui/qopenglfunctions_es2.h>
#include <QtGui/qopenglpaintdevice.h>
#include <QtGui/qopenglpixeltransferoptions.h>
#include <QtGui/qopenglshaderprogram.h>
#include <QtGui/qopengltexture.h>
#include <QtGui/qopengltextureblitter.h>
#include <QtGui/qopengltimerquery.h>
#include <QtGui/qopenglversionfunctions.h>
#include <QtGui/qopenglvertexarrayobject.h>
#include <QtGui/qopenglwindow.h>
